#ifndef STA_CORE_RASPI_SPI_HPP
#define STA_CORE_RASPI_SPI_HPP

#include <sta/config.hpp>
#ifdef STA_PLATFORM_RASPI

#include <sta/spi/device.hpp>
#include <sta/spi/spi.hpp>

namespace sta 
{
    class RaspiSPI : public SPI 
    {
    public:
        RaspiSPI(const char * dev, const SPISettings & settings, Mutex * mutex = nullptr);
        ~RaspiSPI();

        void transfer(uint8_t value) override;
        void transfer16(uint16_t value) override;
        void transfer(const uint8_t * buffer, size_t size) override;
        void transfer(const uint8_t * txBuffer, uint8_t * rxBuffer, size_t size) override;
        void receive(uint8_t * buffer, size_t size) override;

        void fill(uint8_t value, size_t count) override;

        void acquire() override;
        void release() override;

    private:
        char * spidev_;
        int spifd_;
        bool open_;

        void update_setting(int setting, int value);
    };

    class RaspiSPIDevice : SPIDevice 
    {
    public:
        RaspiSPIDevice(RaspiSPI * intf);

        void select() override;

        void deselect() override;
    };
} // namespace sta

#endif // STA_PLATFORM_RASPI

#endif // STA_CORE_RASPI_HPP